var request = require('request');
var deepExtend = require('deep-extend');
var apiUrl = process.env.API_URL || "http://local.otuzbeslik.com:3003/";
var _oblogger_ = require("../src/ob-logger")("hippodamus.api");
var _environment_ = require("../src/environment");
var _errors_ = require("../src/errors");

function get (url, query) {

    var headers = {
        'User-Agent': 'Super Agent/0.0.1',
        'ob-token': _environment_.getVariable("API_TOKEN"),
        'x-apicache-bypass': true,
        'Content-Type': 'application/json'
    };

    var options = {
        url: apiUrl + url,
        method: 'GET',
        headers: headers,
        qs: {}
    };

    if(query) {
        options.qs = deepExtend(options.qs, query);
    }
    
    return _request(options);
}

function post (url, body, query) {

    // Set the headers
    var headers = {
        'User-Agent':       'Super Agent/0.0.1',
        'ob-token': _environment_.getVariable("API_TOKEN"),
        'Content-Type':     'application/json'
    };

    // Configure the request
    var options = {
        url: apiUrl + url,
        method: 'POST',
        headers: headers,
        json: true,
        form: body,
        qs: {}
    };

    if(query) {
        options.qs = deepExtend(options.qs, query);
    }

    return _request(options);
}

function put(url, body, query) {

    // Set the headers
    var headers = {
        'User-Agent': 'Super Agent/0.0.1',
        'ob-token': _environment_.getVariable("API_TOKEN"),
        'Content-Type': 'application/json'
    };

    // Configure the request
    var options = {
        url: apiUrl + url,
        method: 'PUT',
        headers: headers,
        json: true,
        form: body,
        qs: {}
    };

    if (query) {
        options.qs = deepExtend(options.qs, query);
    }

    return _request(options);
}

function postJSON (url, body, query) {

    // Set the headers
    var headers = {
        'User-Agent':       'Super Agent/0.0.1',
        'ob-token': _environment_.getVariable("API_TOKEN"),
        'Content-Type':     'application/json'
    };

    // Configure the request
    var options = {
        url: apiUrl + url,
        method: 'POST',
        headers: headers,
        json: body,
        qs: {}
    };

    if(query) {
        options.qs = deepExtend(options.qs, query);
    }

    return _request(options);
}

function _request(options) {
    
    return new Promise(function (resolve, reject) {

        request(options, function (error, response, body) {

            if (error) {

                return reject({
                    message: "Can not return api result",
                    error: error,
                    data: options.url,
                    code: _errors_.api["can_not_" + options.method.toLowerCase()]
                });
            }

            var parsed;

            try {
                parsed = JSON.parse(body);

            } catch (error) {

                parsed = body;
            }

            if (parsed && parsed.error) {

                resolve({
                    ok: false,
                    error: parsed.error
                });
            } else {

                resolve({
                    ok: response.statusCode >= 200 && response.statusCode < 300,
                    data: parsed
                });
            }
        });
    });
}

function hasInitializationVariables() {

    return _environment_.hasVariables(["API_URL", "API_TOKEN"]);
}

function connect () {

    return new Promise (function (resolve, reject) {

        if (!hasInitializationVariables()) {

            return reject({
                message: "You need to set API_URL and API_TOKEN to get the api working m8."
            });
        }

        get("status")
            .then(function (result) {

                if (result.ok) {

                    resolve(true);
                } else {
                    
                    return reject({
                        message: "Can not status check the api",
                        data: result
                    });
                }
            }).catch (function (error) {

                return reject({
                    message: "Can not connect to the api young fella. Check that shit out.",
                    error: error
                });
            });
    });
}

module.exports = function () {

    return {

        connect: connect,
        get: get,
        post: post,
        put: put,
        postJSON: postJSON,
        getApiUrl: function () {

            return apiUrl;
        }
    }
};