var moment = require("moment");

moment.locale("tr");

var dateFormats = {
    db: "YYYY-MM-DD HH:mm:ss",
    dateTimeNoSeconds: "YYYY-MM-DD HH:mm",
    date: "YYYY-MM-DD",
    time: "HH:mm"
};

var dateRanges = {
    daily: 1,
    weekly: 2,
    monthly: 3
};

var getDateFromDateAndTimeForDb = function (date, time) {

    if (!(date && time)) {

        return null;
    }

    var datePart = moment(date, dateFormats.date);

    if (!datePart.isValid()) {

        datePart = moment(date);

        if (!datePart.isValid()) {

            return null;
        }
    }

    var timePart = moment(time, dateFormats.time);

    if (datePart.isValid() && timePart.isValid()) {

    var result = datePart.set({
        'hour' : timePart.get('hour'),
        'minute'  : timePart.get('minute'), 
        'second' : timePart.get('second')
        });

        return result.isValid()
        ? result.format(dateFormats.db)
        : null;
    }

    /*
    var result = moment(date + " " + time, dateFormats.date + " " + dateFormats.time);

    return result.isValid()
            ? result.format(dateFormats.db)
            : null; */
};

var getDateForDb = function (date) {

    return moment(date).format(dateFormats.db);
};

var dbDateToFormat = function (date, format) {

    var _date = _ensureDate(date, dateFormats.db);

    return _date.format(format);
}

var getStartOf = function (rangeName, date) {

    var d = date ? moment(date) : moment();

    return d.startOf(rangeName).format(dateFormats.db);
};

var getEndOf = function (rangeName, date) {

    var d = date ? moment(date) : moment();
    
    return d.endOf(rangeName).format(dateFormats.db);
};

var getStartOfDay = function (date) { return getStartOf('day', date); };
var getStartOfWeek = function (date) { return getStartOf('week', date); };
var getStartOfMonth = function (date) { return getStartOf('month', date); };
var getEndOfDay = function (date) { return getEndOf('day', date); };
var getEndOfWeek = function (date) { return getEndOf('week', date); };
var getEndOfMonth = function (date) { return getEndOf('month', date); };

var substract = function (date, partName, valueToSubstract) {

    if (!partName) {
        return null
    }

    var _date = date;

    if ("string" == typeof (date)) {

        _date = moment(date);
    }

    var result = null;

    var value = valueToSubstract || 0;

    if (_date) {

        var options = {};
        options[partName] = value;

        result = _date.subtract(options)
    }

    return result ? result.format(dateFormats.db) : null;
}

var subtractDays = function (date, days) {

    return substract(date, "days", days);
}

var subtractHours = function (date, hours) {

    return substract(date, "hours", hours);
}

var getMinDateForRange = function (range) {

    if (dateRanges.daily == range) {

        return getStartOfDay();
    }  

    if (dateRanges.weekly == range) {
        
        return getStartOfWeek();
    }  

    if (dateRanges.monthly == range) {
        
        return getStartOfMonth();
    }  
}

var getMaxDateForRange = function (range) {

    if (dateRanges.daily == range) {

        return getEndOfDay();
    }  

    if (dateRanges.weekly == range) {
        
        return getEndOfWeek();
    }  

    if (dateRanges.monthly == range) {
        
        return getEndOfMonth();
    }  
}

var now = function (format) {

    format = format || dateFormats.db;
    return moment().format(format);
}

var formatDate = function (date, format) {
    
    return format 
            ? moment(date).format(format) 
            : moment(date).format();
};

var formatUTCDate = function (date, format) {
    
    return format 
            ? moment.utc(date).format(format) 
            : moment.utc(date).format();
};

var formatUTCDateForDB = function (date) {

    return !isInDBFormat(date) 
            ? formatUTCDate(date, dateFormats.db)
            : date;
}

var _ensureDate = function (date, initialFormat) {

    var _date = date;

    if ("string" == typeof(date)) {

        if (initialFormat) {

            _date = moment(date, initialFormat);
        } else {

            _date = moment(date);
        }
    }

    return _date;
}
var addDays = function (date, days) {
    
    var _date = _ensureDate(date);

    var result = null;
    
    var _days = days || 0;

    if (_date) {

        result = _date.add(_days, 'd')
    }

    return result ? result.format(dateFormats.db) : null;
};

var dateIsSameOrAfter = function (date, dateToCompare) {

    var _date = _ensureDate(date);
    var _dateToCompare = _ensureDate(dateToCompare);

    return _date.isSameOrAfter(_dateToCompare);
};

var isInDBFormat = function (date) {

    return moment(date, dateFormats.db).format(dateFormats.db) === date;
}

module.exports = {

    now: now,
    dateIsSameOrAfter: dateIsSameOrAfter,
    isInDBFormat: isInDBFormat,
    dbDateToFormat: dbDateToFormat,
    formatDate: formatDate,
    formatUTCDate: formatUTCDate,
    formatUTCDateForDB: formatUTCDateForDB,
    dateFormats: dateFormats,
    addDays: addDays,
    substract: substract,
    subtractHours: subtractHours,
    subtractDays: subtractDays,
	getMinDateForRange: getMinDateForRange,
    getMaxDateForRange: getMaxDateForRange,
    getStartOfDay: getStartOfDay,
    getEndOfDay: getEndOfDay,
    getStartOfWeek: getStartOfWeek,
    getEndOfWeek: getEndOfWeek,
    getStartOfMonth: getStartOfMonth,
    getEndOfMonth: getEndOfMonth,
    getDateForDb: getDateForDb,
    getDateFromDateAndTimeForDb: getDateFromDateAndTimeForDb
};