var mysql = require('mysql');
var moment = require('moment');
var _oblogger_ = require('./ob-logger')("hippodamus.dbmanager");

var dbConfig = {														
	host: process.env.MYSQL_HOST,
	user: process.env.MYSQL_USER,
	password: process.env.MYSQL_PASS,
	database: process.env.MYSQL_SCHEMA,
	connectionLimit: 20,
	dateStrings: true,
	connectTimeout: 30000,											
	multipleStatements: true
};

var pool = mysql.createPool(dbConfig);

function ensureQueryIsInSafeMode(query) {

	var result = query.trim();
	var loweredResult = result.toLowerCase();

	if(-1 != loweredResult.indexOf("delete ") 
	|| -1 != loweredResult.indexOf("update ") 
	|| -1 != loweredResult.indexOf("insert ")) {

		if(result[result.length - 1] != ";") {
	
			result += ";";
		}
		
		result = "SET SQL_SAFE_UPDATES = 0; " + result + " SET SQL_SAFE_UPDATES = 1;";
	}

	return result;
}

function runQuery (query, callback, user) {

	if(!query || '' == query.trim()) {

		_oblogger_.error("Tried to execute empty query");

		return callback();
	}

	try {

		pool.getConnection(function(err, con){

			if(err) {

				_oblogger_.error(err, "There was an error while creating db connection");
				
				con.release();

				return callback();
			}

			var saferQuery = ensureQueryIsInSafeMode(query);
			con.query(saferQuery, function(err,result) {

				con.release();

				if (err) {
					
					_oblogger_.error(err, "Can not run sql query");
					_oblogger_.error(saferQuery);

					callback();
				} else {

					callback(result);
				}
			});
		});
	} catch(err) {

		_oblogger_.error(err, "Something happened while running sql query");
		callback();
	}
}

function runInsertQuery(query, record, callback, user) {

	if(!query || '' == query.trim()) {

		_oblogger_.error("Tried to execute empty query");

		return callback();
	}

	try {	
		
		delete record._csrf;
		record.created = moment().format("YYYY-MM-DD HH:mm:ss");

		pool.getConnection(function(err, con){

			if(err) {

				_oblogger_.error(err, "There was an error while creating db connection");

				con.release();

				return callback();
			}

			con.query(query, record, function(err, result) {

				con.release();

				if (err) {

					_oblogger_.error(err, "Can not run insert query");
					_oblogger_.error(query);

					callback();
				} else {

					callback(result);
				}
			});
		});
	} catch(err) {

		_oblogger_.error(err, "There was an error while running insert query");

		return callback();
	}
}

function runUpdateQuery(query, values, callback, user) {

	if(!query || '' == query.trim()) {

		_oblogger_.error("Tried to execute empty query");

		return callback();
	}

	try {	
		//sleeper.sleep(1);

		pool.getConnection(function(err, con){

			if(err) {

				_oblogger_.error(err, "There was an error while creating db connection");
				con.release();

				return callback();
			}

			con.query(query, values, function(err,result) {

				con.release();

				if (err) {

					_oblogger_.error(err, "There was an error while creating db connection");

					return callback();
				} else {
				
					callback(result);
				}
			});
		});
	} catch(err) {

		_oblogger_.error(err, "There was an error while running update query");

		return callback();
	}
}

function getEqualsStatementForSQL(key, value) {

	var result = [];

	result.push(key + "= ?");

	return result.join("");
}

function generateUpdateQuery(tableName, setValues, whereCriterias) {

	if (!setValues) return;
	if (!whereCriterias) return;

	var resultSQL = [];
	var values = Object.keys(setValues).map(function(key) {
	    return setValues[key];
	});

	//Set updated
	//setValues.updated = moment().format("YYYY-MM-DD HH:mm:ss");

	// Add table name
	resultSQL.push("UPDATE " + tableName);

	// Add set values
	var numberOfSetValues = Object.keys(setValues).length;
	var index = 0;

	resultSQL.push (" SET ");
	for (var setValue in setValues) {

		var value = setValues[setValue];
		resultSQL.push (getEqualsStatementForSQL(setValue, value));

		if (++index != numberOfSetValues) {

			resultSQL.push (", ");
		}
	}

	// Add where criterias
	resultSQL.push (" WHERE ");

	var numberOfWhereCriterias = Object.keys(whereCriterias).length;
	index = 0;
	for (var whereCriteria in whereCriterias) {

		var criteria = whereCriterias[whereCriteria];
		resultSQL.push (getEqualsStatementForSQL(whereCriteria, criteria));

		values.push(criteria);

		if (++index != numberOfWhereCriterias) {

			resultSQL.push (" AND ");
		}
	}

	return {
		query: resultSQL.join(''),
		values: values
	};
}

module.exports = {
	runQuery: runQuery,
	runInsertQuery: runInsertQuery,
	runUpdateQuery: runUpdateQuery,
	generateUpdateQuery: generateUpdateQuery
};