var _ = require("underscore");

var environments = {
    PRODUCTION: ["production"],
    DEVELOPMENT: ["development"],
    TEST: ["test"]        
};

module.exports = {

    environments: environments,

    setVariable: function (key, value) {
    
        process.env[key] = value;
    },

    clearVariable: function (key) {

        this.setVariable(key, "");
    },

    getVariable: function (key) {

        return process.env[key];
    },

    hasVariable: function (key) {

        return !_.isUndefined(process.env[key]) 
                && !_.isEmpty(process.env[key]) 
                && !_.isNull(process.env[key]);
    },

    hasVariables: function (keys) {

        if (_.isEmpty(keys)) { return false; }
        
        return _.every(keys, this.hasVariable);
    },

    setAsTest: function () {
        
        process.env.NODE_ENV = environments.TEST;    
    },

    setAsDevelopment: function () {
        
        process.env.NODE_ENV = environments.DEVELOPMENT;    
    },

    setAsProduction: function () {
        
        process.env.NODE_ENV = environments.PRODUCTION;    
    },

    isProduction: function () {

        return environments.PRODUCTION.includes(process.env.NODE_ENV);
    },

    isTest: function () {

        return environments.TEST.includes(process.env.NODE_ENV);
    },

    isDevelopment: function () {

        return environments.DEVELOPMENT.includes(process.env.NODE_ENV);
    },

    isDefined: function () {

        var nodeEnv = process.env.NODE_ENV;

        return "null" != nodeEnv 
                && "undefined" != nodeEnv 
                && !_.isNull(nodeEnv) 
                && !_.isUndefined(nodeEnv);
    },

    getEnv: function() {
        return process.env.NODE_ENV;
    }
};