var fs = require("fs");
var _text_ = require("./text");
var md5 = require('md5');

function readFileInBase64Sync(filePath) {

    // read binary data
    var bitmap = fs.readFileSync(filePath);
    // convert binary data to base64 encoded string
    return new Buffer(bitmap).toString('base64');
}

function decodeBase64StringAsBuffer (b64string) {

    return Buffer.from(b64string, 'base64')
}

function readFileSync(filePath) {

    return fs.createReadStream(filePath);
}

function getFilesizeInBytes(file) {

	var stats = fs.statSync(file.path);
    var fileSizeInBytes = stats ? stats.size : 0;
  
	return fileSizeInBytes;
}

var getFileExtension = function (file) {

    var originalFilename = file.originalFilename;
    var fileNameParts = originalFilename.split(".");
    return fileNameParts[fileNameParts.length - 1];
};

function getFileContentType(file) {

    if (!file || !(file.name || file.extension)) {
        return null;
    }
    
    var rc = 'application/octet-stream';
    var extension = file.extension;
    
    if (!extension && file.name) {

        var fileName = file.name.toLowerCase();
        var fileNameParts = fileName.split(".");
        extension = fileNameParts[fileNameParts.length - 1];
    } 

    if (!extension) {
        return null;
    }

    switch(extension) {
        case "html": {
            rc = "text/html";
            break;
        }
        case "css": {
            rc = "text/css";
            break;
        }
        case "json": {
            rc = "application/json";
            break;
        }
        case "js": {
            rc = "application/x-javascript";
            break;
        }
        case "png": {
            rc = "image/png";
            break;
        }
        case "jpg": {
            rc = "image/jpg";
            break;
        }
        default: { 
            rc = 'application/octet-stream'
            break;
        }
    }
  
    return rc;
}

function writeObject(filePath, obj) {

    return new Promise (function (resolve, reject) {

        var objString = _text_.stringify(obj);

        if(objString) {
    
            fs.writeFile(filePath, objString, 'utf8', function (err){

                if (err) {

                    reject(err);
                } else {

                    resolve(filePath); 
                } 
            });
        } else {

            resolve();
        }
    });  
}

function writeObjectSync(filePath, obj) {

    var objString = _text_.stringify(obj);

    if(objString) {

        return fs.writeFileSync(filePath, objString, 'utf8');
    }
}

module.exports = {
    writeObject: writeObject,
    writeObjectSync: writeObjectSync,
    readFileInBase64Sync: readFileInBase64Sync,
    readFileSync: readFileSync,
    decodeBase64StringAsBuffer: decodeBase64StringAsBuffer,
    getFilesizeInBytes: getFilesizeInBytes,
    getFileExtension: getFileExtension,
    getFileContentType: getFileContentType
};
