var request = require('request');
var _object_ = require('./object');
var _errors_ = require('./errors');
var promiseSerial = require('./promises').promiseSerial;
var _ = require("underscore");
var _oblogger_ = require('./ob-logger')("hippodamus.grabby");

var bud = require('basic-instagram-user-details');
var scrape = require('html-metadata');

function getFacebookIdRequestUrl (facebookName) {

    var fbAccessToken = process.env.FACEBOOK_ACCESS_TOKEN;

    if (fbAccessToken && facebookName) {

        return "https://graph.facebook.com/" + facebookName + "?access_token=" + fbAccessToken;
    }
}

function parseOAuthApiResponse (body) {

    var result
        , key
        , value
        , split;

    result = {};
    body = body.split('&');
    for(var key=0, l=body.length; key<l; key++) {
        split = body[key].split('=');
        if(split.length === 2) {
            value = split[1];
            if(!isNaN(value)) {
                result[split[0]] = parseInt(value);
            } else {
                result[split[0]] = value;
            }
        }
    }

    return result;
}

var cleanupInstagramHandle = function (instagramHandle) {

    if (!instagramHandle) { return null; }
    
    instagramHandle = instagramHandle.replace("https://www.instagram.com/", "");

    return instagramHandle.trim();
};

var getInstagramIdByHandle = function (instagramHandle) {
        
    return new Promise(function (resolve, reject) {

        if (!instagramHandle) {

            return resolve(null);
        }

        instagramHandle = cleanupInstagramHandle(instagramHandle);

        return bud(instagramHandle, 'id').then(function (response) {
            
            if (response 
                && response.data
                && "string" == typeof(response.data)
                //Error messages also come as string and ids dont have spaces
                && !(/\s/.test(response.data.trim()))) {

                resolve(response.data)
            } else {

                resolve(null);
            }
        }).catch(reject);
    });
};

var getFacebookIdByUrl = function (facebookUrl) {

    return new Promise(function (resolve, reject) {

       // var facebookName = getFacebookNameFromUrl(facebookUrl);
        var facebookIdRequestUrl = getFacebookIdRequestUrl(facebookUrl);

        if (!facebookIdRequestUrl) {

            return resolve(null);
        }

        request(facebookIdRequestUrl, function (error, response, body) {
    
            if(error !== null) {

                if(error === Object(error) && _object_.hasProperty(error, 'error')) {
                    return reject(error);
                }

                return reject({
                    error:error
                });
            }
    
            var result;

            if(response && response.statusCode === 200 &&
                response.headers && /.*text\/plain.*/.test(response.headers['content-type'])) {

                    result = parseOAuthApiResponse(body);
            } else {

                try {
                    result = JSON.parse(body);
                } catch (ex) {
                  // sometimes FB is has API errors that return HTML and a message
                  // of "Sorry, something went wrong". These are infrequent and unpredictable but
                  // let's not let them blow up our application.
                    reject({ 
                        error: {
                            code: _errors_.json.can_not_parse,
                            error: ex
                        }
                    });
                }
            }

            if (result 
                && result.id
                && facebookUrl != result.id) {

                resolve(result.id);
            } else {
                
                resolve(null);
            }
        });
    });
};

var getFoursquareIdByUrl = function (foursquareUrl) {

    return new Promise(function (resolve, reject) {

        scrape(foursquareUrl).then(function(metadata2){
                
            if (metadata2 
                && metadata2.twitter
                && metadata2.twitter.app
                && metadata2.twitter.app.url
                && metadata2.twitter.app.url.iphone) {

                    var urlParts = metadata2.twitter.app.url.iphone.split("/");
                    var venue_id = urlParts[urlParts.length - 1];
                    resolve(venue_id);
            } else {

                resolve(null);
            }
        }).catch(function () {

            reject(null);
        });
    });
};

module.exports = {
    
    getThirdPartyInfoForEntity: function (entity) {

        return new Promise(function (resolve, reject) {

            if (!entity) {
                
                return resolve(null);
            }

            var thirdPartyFetchPromises = [];
            var thirdPartyFetchKeys = [];
    
            if (entity.facebook_url
                && !entity.facebook_id) {
    
                thirdPartyFetchKeys.push("facebook_id");
                thirdPartyFetchPromises.push(function () {

                    return getFacebookIdByUrl(entity.facebook_url);
                });
            }
    
            var instagramKey = entity.instagram_handle || entity.instagram_url;
    
            if (instagramKey
                && !entity.instagram_id) {
    
                thirdPartyFetchKeys.push("instagram_id");
                thirdPartyFetchPromises.push(function () {

                    return getInstagramIdByHandle(instagramKey);
                });
            }
    
            if (entity.foursquare_url
                && !entity.foursquare_id) {
    
                thirdPartyFetchKeys.push("foursquare_id");
                thirdPartyFetchPromises.push(function () {

                    return getFoursquareIdByUrl(entity.foursquare_url);
                });
            }

            if (_.isEmpty(thirdPartyFetchPromises)) {
    
                return resolve(null);
            } else {

                promiseSerial(thirdPartyFetchPromises)
                    .then(function (results) {

                        var resultInfo = {};

                        _.each(results, function (result, index) {

                            if (result) {

                                resultInfo[thirdPartyFetchKeys[index]] = result;
                            }
                        });

                        resolve(resultInfo);
                    }).catch(reject);
            }
        });
    },

    getInstagramIdByHandle: getInstagramIdByHandle,
    getFacebookIdByUrl: getFacebookIdByUrl,
    getFoursquareIdByUrl: getFoursquareIdByUrl
};
