var _ = require("underscore");
var _util_ = require("../../util");

var groupedLocationProperties = [
    {
		"header_key": "card_payments",
		"properties": [
            {
				"key": "has_card_payments",
				"type": "boolean"
			},
			{
				"key": "has_visa_card_payments",
				"type": "boolean"
			},
			{
				"key": "has_mastercard_payments",
				"type": "boolean"
			},
			{
				"key": "has_american_express_card_payments",
				"type": "boolean"
			}
		]
	},

	{
		"header_key": "food",
		"properties": [{
				"key": "has_breakfast",
				"type": "boolean"
			},
			{
				"key": "has_lunch",
				"type": "boolean"
			},
			{
				"key": "has_brunch",
				"type": "boolean"
			},
			{
				"key": "has_wine",
				"type": "boolean"
			},
			{
				"key": "has_cocktails",
				"type": "boolean"
			},
			{
				"key": "has_full_bar",
				"type": "boolean"
			},
			{
				"key": "has_cocktails",
				"type": "boolean"
			},
			{
				"key": "has_restaurant",
				"type": "boolean"
			},
			{
				"key": "has_picnic_units",
				"type": "boolean"
			},
			{
				"key": "has_desserts",
				"type": "boolean"
			},
			{
				"key": "has_barbeque",
				"type": "boolean"
			},
			{
				"key": "has_buffet",
				"type": "boolean"
			},
			{
				"key": "has_happy_hour",
				"type": "boolean"
			},
			{
				"key": "has_delivery",
				"type": "boolean"
			},
			{
				"key": "has_drinking_water",
				"type": "boolean"
			}
		]
	},

	{
		"header_key": "rating",
		"properties": [

			{
				"key": "price_tier",
				"min": 1,
				"max": 3
			},
			{
				"key": "fs_rating",
				"min": 0,
				"max": 10
			},
			{
				"key": "fb_rating_count",
				"min": 0,
				"max": 5
			},
			{
				"key": "google_rating",
				"min": 0,
				"max": 5
			},
			{
				"key": "rating",
				"min": 0,
				"max": 10
			}
		]
	},

	{
		"header_key": "accommodation",
		"properties": [

			{
				"key": "has_shower",
				"type": "boolean"
			},

			{
				"key": "has_autopark",
				"type": "boolean"
			},
			{
				"key": "has_laundry",
				"type": "boolean"
			},
			{
				"key": "has_kitchen",
				"type": "boolean"
			},
			{
				"key": "has_usable_refrigerator",
				"type": "boolean"
			},

			{
				"key": "has_tent_rental",
				"type": "boolean"
			},
			{
				"key": "has_bungalows",
				"type": "boolean"
			},
			{
				"key": "has_van_waste_water_discharge",
				"type": "boolean"
			},
			{
				"key": "has_electric",
				"type": "boolean"
			},
			{
				"key": "takes_reservations",
				"type": "boolean"
			},
			{
				"key": "takes_only_group_reservations",
				"type": "boolean"
			}
		]
	},

	{
		"header_key": "additional",
		"properties": [

			{
				"key": "dressing_room",
				"type": "boolean"
			},
			{
				"key": "has_music",
				"type": "boolean"
			},
			{
				"key": "is_bicycle_friendly",
				"type": "boolean"
			},
			{
				"key": "is_pet_friendly",
				"type": "boolean"
			},
			{
				"key": "closed",
				"type": "boolean"
			},
			{
				"key": "has_beaches",
				"type": "boolean"
			}
		]
	}
];


module.exports = {

    groupProperties: function (locationProperties) {

        if (_.isEmpty(locationProperties)) {

            return null;
        }

        var result = [];

        _.each(groupedLocationProperties, function (groupedLocationProperty) {

            var group = {
                header_key: groupedLocationProperty.header_key,
                properties: []
            };

            _.each(groupedLocationProperty.properties, function (property) {

                var propertyValue = locationProperties[property.key];

                if (!_.isNull(propertyValue) 
                    && !_.isUndefined(propertyValue)) {

						var propertyItem = _.clone(property);

                        if ("boolean" == property.type) {

                            propertyItem.value = _util_.getBooleanValue(propertyValue);
                        } else {

                            propertyItem.value = propertyValue;
                        }

						group.properties.push(propertyItem);
                }
            });

            result.push(group);
        });

        return result;
    }
};

