var Nightmare = require('nightmare');
var deepExtend = require('deep-extend');

var USER_AGENTS = [
    'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:51.0) Gecko/20100101 Firefox/51.0',
    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.110 Safari/537.36',
    'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.76 Safari/537.36',
];  

module.exports = function (options) {
        
    var nightmareConfig = deepExtend({}, { 
        show: false,
        //waitTimeout: 1000, // in ms
        webPreferences: {
            images: false
        } 
    }, options || {});

    Nightmare.action('waitUntilNetworkIdle',
        // The first callback defines the action on Electron's end,
        // making some internal objects available.
        function (name, options, parent, win, renderer, done) {
    
        // `parent` is Electron's reference to the object that
        // passes messages between Electron and Nightmare.
        parent.respondTo('waitUntilNetworkIdle', (waitTime, done) => {
            let lastRequestTime = Date.now();
    
            // win.webContents allows us to control the internal
            // Electron BrowserWindow instance.
            win.webContents.on('did-get-response-details', () => {
            lastRequestTime = Date.now();
            });
    
            const check = () => {
            const now = Date.now();
            const elapsedTime = now - lastRequestTime;
            if (elapsedTime >= waitTime) {
                done(); // Complete the action.
            } else {
                setTimeout(check, waitTime - elapsedTime);
            }
            }
            setTimeout(check, waitTime);
        });
    
        done(); // Complete the action's *creation*.
        },
        // The second callback runs on Nightmare's end and determines
        // the action's interface.
        function (waitTime, done) {
        // This is necessary because the action will only work if
        // action arguments are specified before `done`, and because
        // we wish to support calls without arguments.
        if (!done) {
            done = waitTime;
            waitTime = 500;
        }
    
        // `this.child` is Nightmare's reference to the object that
        // passes messages between Electron and Nightmare.
        this.child.call('waitUntilNetworkIdle', waitTime, done);
    });
    
    var nightmare = Nightmare(nightmareConfig);

    return {
        getNightmare: function () {
    
            var ua = USER_AGENTS[Math.floor(Math.random() * USER_AGENTS.length)];
        
            return nightmare.useragent(ua);
        }
    };
};