var _ = require("underscore");

module.exports = {
    
    promiseSerial: function (funcs) {

        if(!funcs) {
            return Promise.reject;
        }

        return funcs.reduce((promise, func) =>
            promise.then(result => func()
                .then(Array.prototype.concat.bind(result))
                .catch(Array.prototype.concat.bind(result))
            ),
            Promise.resolve([]));
    },

    mergePromiseResults: function (promiseResults) {

        if (promiseResults && promiseResults.length) {

            var result = [];

            _.each(promiseResults, function (promiseResult) {

                result = [].concat(result, promiseResult.data);
            })

            return result;
        }
    },

    promisify: function (fn) { 
        
        return function () {
            
            var args = [];

            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }

            return new Promise(function (resolve, reject) {

                var result = null;

                try {

                    result = fn.apply(void 0, args);
                    resolve(result);
                } catch (error) {

                    reject(error);
                } 
            });
        }; 
    }    
};