var _images = require("./images");
var _promiseSerial = require("./promises").promiseSerial;
var _ = require("underscore");
var _obLogger_ = require('./ob-logger')("hippodamus.request");
var _file = require('./file');
var _array = require('./array');
var uniqid = require('uniqid');

function requestHasImages(req) {

    if (!req.files || 0 == Object.keys(req.files).length) {
      return false;
    }

    for (var fileName in req.files) {

      var file = req.files[fileName];

      if(0 != file.size) {
        return true;
      }
    }

    return false;
}

function getFilesFromRequest (req, options) {

    if (requestHasImages(req)) {                                

        var files = Object.keys(req.files).map(function(key) {

            return req.files[key];
        });  

        var result = _array.flatten(files);

        for(var i = 0; i < result.length; i++) {

            result[i].name = uniqid();
        }

        return result;
    }
}

function getImageFromFile (file, options) {

    return new Promise(function (resolve, reject) {

        _images.getImageFromFile(file, options)
            .then(function (stream) {
                
                var image = {
                    name: file.name,
                    fieldName: file.fieldName,
                    originalFileName: file.originalFilename,
                    stream: stream,
                    size: file.size,
                    prefix: file.prefix,
                    contentLength: _file.getFilesizeInBytes(file),
                    contentType: _file.getFileContentType(file),
                    extension: _file.getFileExtension(file)
                };

                resolve (image);                                                
            })
            .catch(function (error) {

                _obLogger_.error(error, "Could not get image from file");
                reject(error);
            });
    });
}

function getImagesFromFiles (files, options) {

    return new Promise(function (resolve, reject) {

        if (files) {
                
            var imagePromises = files.map(file => () => {
                
                return getImageFromFile(file, options);
            });
            
            _promiseSerial(imagePromises)
                .then(function (images) {

                    resolve(images);
                })
                .catch(function (error) {

                    _obLogger_.error(error, "Could not get images from files");
                    reject(error);
                });
        }
        else {
            resolve(null);
        }
    });
}

function getImagesFromRequest(req, options) {

    return new Promise(function (resolve, reject) {

        var files = getFilesFromRequest(req, options);
        
        if (files) {                                         
            
            _images.buildResizedImages(files, options.sizes)
                .then(function (builtImages) {
        
                    if (!_.isEmpty(builtImages)) {

                        Array.prototype.push.apply(files, builtImages);
                    }

                    getImagesFromFiles(files, options)
                        .then(function (images) {

                            resolve(images);
                        })
                        .catch(function (error) {

                            _obLogger_.error(error, "Could not get images from request");
                            reject(error);
                        });
                })
                .catch(function (error) {

                    _obLogger_(error, "Could not build resized images");
                    reject(error);
                });
        } else {
            
            resolve();
        }
    });
}

module.exports = {

    has:  {
        images: requestHasImages 
    },

    files: {
        getImagesFromRequest: getImagesFromRequest,
        getAll: getFilesFromRequest
    }
};