var AWS = require('aws-sdk');
var _oblogger = require("./ob-logger")("S3 Manager");
var _file = require("./file");
var _images_ = require("./images");
var _environment_ = require("./environment");
var buildUrl = require('build-url');
var ENTITY_TYPES = require("./core").ENTITY_TYPES;

//AWS.config.loadFromPath('credentials/aws-config.json');
AWS.config.region = process.env.AWS_REGION; //'eu-central-1';
AWS.config.accessKeyId = process.env.AWS_ACCESS_KEY_ID;
AWS.config.secretAccessKey = process.env.AWS_SECRET_ACCESS_KEY;

/*
if(!process.env.AWS_REGION
|| !process.env.AWS_ACCESS_KEY_ID
|| !process.env.AWS_SECRET_ACCESS_KEY
|| !process.env.AWS_S3_URL
|| !process.env.AWS_S3_BUCKET) {

  _oblogger.error("You need to specify AWS_REGION, AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY, AWS_S3_URL, AWS_S3_BUCKET to use hippodamus.s3-manager");
} */

var baseS3Url = process.env.AWS_S3_URL; // "https://otuzbeslik.s3.amazonaws.com/";

function environmentIsSet() {
    
  return process.env.AWS_REGION
      && process.env.AWS_ACCESS_KEY_ID
      && process.env.AWS_SECRET_ACCESS_KEY
      && process.env.AWS_S3_URL
      && process.env.AWS_S3_BUCKET;
} 

function uploadFile(file) {

  return new Promise (function (resolve, reject) {

    if (!environmentIsSet()) {

      return reject({
        message: "You need to specify AWS_REGION, AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY, AWS_S3_URL, AWS_S3_BUCKET to use hippodamus.s3-manager"
      })
    }

    var s3 = new AWS.S3();
    var fileName = file.name;
    var s3Key = file.s3BucketPath + '/' + fileName + "." + file.extension;
  
    _oblogger.info('Uploading file [' + fileName + '] as [' + file.contentType + '] to bucket ' + file.s3BucketPath);
  
    var s3object = {
      ACL: 'public-read',
      Bucket: process.env.AWS_S3_BUCKET,//'otuzbeslik',
      Key: s3Key,
      Body: _file.decodeBase64StringAsBuffer(file.stream),
      ContentLength: file.contentLength,
      ContentType: file.contentType
    };

    s3.putObject(s3object, function(error, response) {
      
      if (error){
  
        reject(error);
      } else {
    
        resolve({
          image_url: s3Key,
          fieldName: file.fieldName,
          size_prefix: file.size && file.size.prefix 
                        ? file.size.prefix 
                        : _images_.IMAGESIZES.REGULAR.prefix
        }); 
      }
    });
  });
}

var getAbsoluteCdnUrl = function (relativeUrl) {

  var awsS3Url = getAWSS3Url();
  var result = null;

  if (awsS3Url) {

    result = buildUrl(awsS3Url, {
      path: relativeUrl
    });

  } 

  return result;
};

var stripTrailingSlash = (str) => {

  return str.endsWith('/') ?
      str.slice(0, -1) :
      str;
};

var getAWSS3Url = function () {

  var url = _environment_.isProduction()
        ? _environment_.getVariable("AWS_S3_URL")
        : (
          _environment_.isDevelopment() 
            ? _environment_.getVariable("AWS_S3_URL_DEV")
            : (
              _environment_.isTest()
                ? _environment_.getVariable("AWS_S3_URL_TEST")
                : null
            )
          );

  return stripTrailingSlash(url);
}

var getS3BucketPathByEntityType = function (type) {

  switch (type) {
      case ENTITY_TYPES.BANNER:
        return "img/banner";
      case ENTITY_TYPES.LOCATION:
          return "img/mekan";
      case ENTITY_TYPES.ARTIST:
          return "img/kisi";
      case ENTITY_TYPES.GROUP:
          return "img/grup";
      case ENTITY_TYPES.EVENT:
          return "img/etkinlik";
      case ENTITY_TYPES.POST:
          return "img/yazi";
      case ENTITY_TYPES.PARTNER:
        return "img/partner";
      case ENTITY_TYPES.OPPORTUNITY:
        return "img/firsat";
      case ENTITY_TYPES.GALLERY:
        return "img/galeri";
      case ENTITY_TYPES.MERCHANDISE:
        return "img/urun";
      default:
          return null;
  }
};

var getS3PathForFile = function (s3BucketPath, name, extension) {

  if (s3BucketPath && name && extension) {

    return s3BucketPath + '/' + name + "." + extension;
  }
}

module.exports = {
	uploadFile: uploadFile,
  getAbsoluteCdnUrl: getAbsoluteCdnUrl,
  getAWSS3Url: getAWSS3Url,
  getS3PathForFile: getS3PathForFile,
  getS3BucketPathByEntityType: getS3BucketPathByEntityType
};