'use strict';
const LosslessJSON = require('lossless-json');
const fixUtf8 = require('fix-utf8')
var h2p = require('html2plaintext');
var lodash = require("lodash");

var stringify = function (obj) {

    if (!obj) {
        return null;
    }

    var cache = [];
    var objectString = LosslessJSON.stringify(obj, function(key, value) {

        if (typeof value === 'object' && value !== null) {
            if (cache.indexOf(value) !== -1) {
                // Duplicate reference found
                try {
                    // If this value does not reference a parent it can be deduped
                    return LosslessJSON.parse(LosslessJSON.stringify(value));
                } catch (error) {
                    // discard key if value cannot be deduped
                    return;
                }
            }
            // Store value in our collection
            cache.push(value);
        }
        return value;
    }, 4);
    
    cache = null; // Enable garbage collection

    return objectString;
};

function numbersOnly (text) {

    return text ? text.replace(/[^0-9\.]+/g, "") : null;
}

function textOnly (text, options) {

    if (!text) {
        return ""
    }
    
    var result = h2p(text);
    result = fixUtf8(result)

    if (options) {

        if (options.noSpace) {

            result = result.replace( /\s\s+/g, ' ' );
        }

        if (options.noPunctuationMarks) {

            result = lodash.words(result).join(' ');
        }
    }

    return result;
}

function fixText (text){
    
    return fixUtf8(text);
}

function replaceAll (target, search, replacement) {
    
    return target.replace(new RegExp(search, 'g'), replacement);
};

module.exports = {
    
    replaceAll: replaceAll,
    fixText: fixText,
    stringify: stringify,
    textOnly: textOnly,
    numbersOnly: numbersOnly
};