var _date_ = require("../../src/date");
var _environment_ = require("../../src/environment");
var moment = require ("moment");

var chai = require('chai');
var expect  = chai.expect;

describe("Date", function() {

    before(function(done) {

        _environment_.setAsTest();
        done();
    });

    after(function(done) {

        _environment_.setAsTest();
        done();
    });

    it('can create date for db from date and time', function(done) {
    
        //_environment_.setAsDevelopment();

        expect(_date_.getDateFromDateAndTimeForDb("2017-01-01", "10:20")).to.equal("2017-01-01 10:20:00");
        expect(_date_.getDateFromDateAndTimeForDb("2017-10-04T00:00:00.000Z", "03:00")).to.equal("2017-10-04 03:00:00");
        
        done();
    });

    it('can subtract days from date', function(done) {

        expect(_date_.subtractDays()).to.be.null;

        var now = _date_.now();
        expect(_date_.subtractDays(now)).to.be.eql(now);

        var date = moment().year(2018).month(7).date(10).hour(0).minute(0).second(0);
        var pastDate = _date_.subtractDays(date, 7);

        expect(pastDate).to.be.eql("2018-08-03 00:00:00");
        
        done();
    });


    it('can subtract hours from date', function (done) {

        expect(_date_.subtractDays()).to.be.null;

        var now = _date_.now();
        expect(_date_.subtractHours(now)).to.be.eql(now);

        var date = moment().year(2018).month(7).date(10).hour(3).minute(0).second(0);
        var pastDate = _date_.subtractHours(date, 3);

        expect(pastDate).to.be.eql("2018-08-10 00:00:00");

        done();
    });


    it('can add days from date', function(done) {

        expect(_date_.addDays()).to.be.null;

        var now = _date_.now();
        expect(_date_.addDays(now)).to.be.eql(now);

        var date = moment().year(2018).month(7).date(10).hour(0).minute(0).second(0);
        var futureDate = _date_.addDays(date, 7);

        expect(futureDate).to.be.eql("2018-08-17 00:00:00");
        
        done();
    });

    it('can check if date is same or after another date', function(done) {

        var date = moment().year(2018).month(7).date(10).hour(0).minute(0).second(0);
        var dateSame = moment().year(2018).month(7).date(10).hour(0).minute(0).second(0);
        var dateBefore = moment().year(2018).month(7).date(9).hour(0).minute(0).second(0);
        var dateAfter = moment().year(2018).month(7).date(10).hour(1).minute(0).second(0);

        expect(_date_.dateIsSameOrAfter(date, dateAfter)).to.be.false;
        expect(_date_.dateIsSameOrAfter(date, dateSame)).to.be.true;
        expect(_date_.dateIsSameOrAfter(date, dateBefore)).to.be.true;
        expect(_date_.dateIsSameOrAfter("2019-04-23", "2019-03-23")).to.be.true;
        expect(_date_.dateIsSameOrAfter("2019-04-23", "2019-04-23")).to.be.true;
        expect(_date_.dateIsSameOrAfter("2019-04-23", "2019-05-23")).to.be.false;
        
        done();
    });

    it('can format date', function(done) {

        var date = _date_.formatUTCDateForDB("Sat, 23 Mar 2019 21:05:55 GMT");
       
        expect(date).to.eql("2019-03-23 21:05:55");
        
        done();
    });

    it('can check if date is in db format', function(done) {

        var shouldBeTrue = _date_.isInDBFormat("2019-03-23 21:05:55");
        var shouldBeFalse = _date_.isInDBFormat("Sat, 23 Mar 2019 21:05:55 GMT");
       
        expect(shouldBeTrue).to.be.true;
        expect(shouldBeFalse).to.be.false;
        
        done();
    });
});
    