var _environment_ = require("../../src/environment");

var chai = require('chai');
var expect  = chai.expect;

describe("Environment", function() {

    beforeEach(function(done) {

        _environment_.setAsTest();
        done();
    });

    after(function(done) {

        _environment_.setAsTest();
        done();
    });

    it('can set as development', function(done) {

        _environment_.setAsDevelopment();

        expect(_environment_.isDevelopment()).to.be.true;
        expect(_environment_.isTest()).to.be.false;
        expect(_environment_.isProduction()).to.be.false;

        done();
    });

    it('can set as test', function(done) {

        _environment_.setAsTest();

        expect(_environment_.isTest()).to.be.true;
        expect(_environment_.isDevelopment()).to.be.false;
        expect(_environment_.isProduction()).to.be.false;

        done();
    });

    it('can set as production', function(done) {

        _environment_.setAsProduction();

        expect(_environment_.isProduction()).to.be.true;
        expect(_environment_.isTest()).to.be.false;
        expect(_environment_.isDevelopment()).to.be.false;

        done();
    });

    it('can check if it is defined', function(done) {

        process.env.NODE_ENV = null;

        expect(_environment_.isDefined()).to.be.false;
        
        process.env.NODE_ENV = undefined;

        expect(_environment_.isDefined()).to.be.false;
        
        done();
    });

    it('can set variable', function(done) {

        var testVariableKey = "TEST_VARIABLE_" + Math.random();
        var testVariableValue = "TEST_VALUE";

        _environment_.setVariable(testVariableKey, testVariableValue);

        expect(process.env[testVariableKey]).to.be.eql(testVariableValue);

        done();
    });

    it('can check if variable exists', function(done) {

        var testVariableKey = "TEST_VARIABLE_" + Math.random();
        var testVariableValue = "TEST_VALUE";

        _environment_.setVariable(testVariableKey, testVariableValue);
        expect(_environment_.hasVariable(testVariableKey)).to.be.true;

        expect(_environment_.hasVariable(null)).to.be.false;
        expect(_environment_.hasVariable("")).to.be.false;

        done();
    });

    it('can check if variables exists', function(done) {

        var testVariableKey = "TEST_VARIABLE_" + Math.random();
        var testVariableValue = "TEST_VALUE";

        var testVariable2Key = "TEST_VARIABLE2_" + Math.random();
        var testVariable2Value = "TEST_VALUE2";

        _environment_.setVariable(testVariableKey, testVariableValue);
        _environment_.setVariable(testVariable2Key, testVariable2Value);

        expect(_environment_.hasVariables([testVariableKey, testVariable2Key])).to.be.true;
        expect(_environment_.hasVariables(null)).to.be.false;

        done();
    });


    it('can clear environment variable', function(done) {

        var key = "TEST_VARIABLE_CLEAR_ENV";

        _environment_.setVariable(key, "TEST_VALUE");
        expect(_environment_.hasVariable(key)).to.be.true;

        _environment_.clearVariable(key);
        expect(_environment_.hasVariable(key)).not.to.be.true;

        done();
    });
});
    