var _grabby_ = require("../../src/grabby");
var _environment_ = require("../../src/environment");
var promiseSerial = require("../../src/promises").promiseSerial;
var _ = require("underscore");

var chai = require('chai');
var expect  = chai.expect;

describe("Grabby", function() {

    this.timeout(30000);

    before(function(done) {

        _environment_.setAsTest();
        done();
    });

    after(function(done) {

        _environment_.setAsTest();
        done();
    });

    it('can get facebook id by facebook url', function(done) {

        this.skip();

        var testCases = [{
            input: "https://www.facebook.com/Jacksons-Alsancak-1082751255195606/",
            expectedOutcome: "1082751255195606"
        }];
            
        var testCasePromises = testCases.map(testCase => () => {
    
            return _grabby_.getFacebookIdByUrl(testCase.input);
        });

        promiseSerial(testCasePromises)
            .then(function (results) {

                _.each(results, function (result, index) {
                    
                    expect(result).to.be.eql(testCases[index].expectedOutcome);
                });

                done();
            }).catch(function (error) { 

                console.log(error);
            });

    });

    it('can get instagram id by instagram handle', function(done) {

        this.skip();

        var testCases = [{
            input: "kordonotel",
            expectedOutcome: "1558370972"
        },
        {
            input: "denizrestaurantalsancak",
            expectedOutcome: "3968836138"
        }];
            
        var testCasePromises = testCases.map(testCase => () => {
    
            return _grabby_.getInstagramIdByHandle(testCase.input);
        });

        promiseSerial(testCasePromises)
            .then(function (results) {

                _.each(results, function (result, index) {
                
                    expect(result).to.be.eql(testCases[index].expectedOutcome);
                });

                done();
            }).catch(function (error) { 

                console.log(error);
            });
    });

    it('can handle empty facebook urls', function(done) {

        this.skip();

        var nullTestCases = [null, undefined, ""];
        
        var nullTestCasePromises = nullTestCases.map(nullTestCase => () => {
    
            return _grabby_.getFacebookIdByUrl(nullTestCase);
        });

        promiseSerial(nullTestCasePromises)
            .then(function (results) {

                _.each(results, function (result) {

                    expect(result).to.be.null;
                });
                done();
            }).catch(function (error) { 

            });
    });

    it('can get foursquare id by foursquare url', function(done) {

        this.skip();

        var testCases = [{
            input: "https://tr.foursquare.com/meyhaneendulus",
            expectedOutcome: "4cc9ba91bcb1b1f75c0e118a"
        },
        {
            input: "https://foursquare.com/v/ak%C4%B1n%C4%B1n-yeri/4bd06dde41b9ef3ba503fae5",
            expectedOutcome: "4bd06dde41b9ef3ba503fae5"
        },
        {
            input: "https://tr.foursquare.com/v/aqua-%C5%9Fifne/4c1ce3c08b3aa593b68b995f",
            expectedOutcome: "4c1ce3c08b3aa593b68b995f"
        }];
            
        var testCasePromises = testCases.map(testCase => () => {
    
            return _grabby_.getFoursquareIdByUrl(testCase.input);
        });

        promiseSerial(testCasePromises)
            .then(function (results) {

                _.each(results, function (result, index) {
                
                    expect(result).to.be.eql(testCases[index].expectedOutcome);
                });

                done();
            }).catch(function (error) { 

                console.log(error);
            });
    });

    it('can generate third party info for entity', function(done) {
    
        this.skip();

        var testCases = [
        {
            input: {
                instagram_handle: "https://www.instagram.com/melikecercioglubilgic/",
                foursquare_url: "https://tr.foursquare.com/meyhaneendulus"
            },
            expectedOutcome: {
                instagram_id: "49427295",
                foursquare_id: "4cc9ba91bcb1b1f75c0e118a"
            }
        },
        {
            input: {
                foursquare_url: "https://tr.foursquare.com/meyhaneendulus",
                facebook_url: "https://www.facebook.com/Jacksons-Alsancak-1082751255195606/",
                instagram_url: "kordonotel"
            },
            expectedOutcome: {
                facebook_id: "1082751255195606",
                foursquare_id: "4cc9ba91bcb1b1f75c0e118a",
                instagram_id: "1558370972"
            }
        },
        {
            input: {
                foursquare_url: "https://tr.foursquare.com/meyhaneendulus",
                facebook_url: "https://www.facebook.com/Jacksons-Alsancak-1082751255195606/",
                instagram_handle: "kordonotel"
            },
            expectedOutcome: {
                facebook_id: "1082751255195606",
                foursquare_id: "4cc9ba91bcb1b1f75c0e118a",
                instagram_id: "1558370972"
            }
        }];
            
        var testCasePromises = testCases.map(testCase => () => {
    
            return _grabby_.getThirdPartyInfoForEntity(testCase.input);
        });

        promiseSerial(testCasePromises)
            .then(function (results) {

                _.each(results, function (result, index) {
                
                    expect(result).to.be.eql(testCases[index].expectedOutcome);
                });

                done();
            }).catch(function (error) { 

                console.log(error);
            });
    });

    it('can generate third party for wrong input', function (done) {

        var testCases = [
            {
                input: {
                    facebook_url: "2sddsddd",
                    instagram_handle: "otuzbeslikcom32322323",
                    foursquare_url: "ddfdf"
                },
                expectedOutcome: {
                }
            }];

        var testCasePromises = testCases.map(testCase => () => {

            return _grabby_.getThirdPartyInfoForEntity(testCase.input);
        });

        promiseSerial(testCasePromises)
            .then(function (results) {

                _.each(results, function (result, index) {

                    expect(result).to.be.eql(testCases[index].expectedOutcome);
                    expect(result).to.be.empty;
                });

                done();
            }).catch(function (error) {

                console.log(error);
            });
    });
});
