var _locationViewHelper_ = require("../../../../src/helpers/location/view");
var _environment_ = require("../../../../src/environment");
var _oblogger_ = require("../../../../src/ob-logger")("hippodamus.tests.helpers.location.view");

var chai = require('chai');
var expect  = chai.expect;
var _ = require("underscore");

describe("Location View Helper", function() {

    before(function(done) {

        _environment_.setAsTest();
        done();
    });

    after(function(done) {

        _environment_.setAsTest();
        done();
    });

    it('can handle null values while grouping properties', function(done) {
    
        expect(_locationViewHelper_.groupProperties()).to.be.null;
        expect(_locationViewHelper_.groupProperties(null)).to.be.null;
        expect(_locationViewHelper_.groupProperties(undefined)).to.be.null;

        done();
    });

    it('can set values while grouping properties', function(done) {
    
        var testCase = {

            "has_card_payments": 1,
            "has_beaches": 1,
            "closed": 1,
            "google_rating": 2.8 
        };

        var grouped = _locationViewHelper_.groupProperties(testCase);
        
        _.each(grouped, function (groupedItem) {

            if (!_.isEmpty(groupedItem.properties)) {
  
                _.each(groupedItem.properties, function (property) {

                    expect(property.value).not.to.be.undefined;
                    expect(property.value).not.to.be.null;
                });
            }
        });

        done();
    });

    it('can group boolean values', function(done) {
    
        var booleanTestCases = {

            intValueTrue: { "has_card_payments": 1 },
            intValueFalse: { "has_card_payments": 0 },
            stringValueTrue: { "has_card_payments": "1" },
            stringValueFalse: { "has_card_payments": "0" },
            booleanAsStringValueTrue: { "has_card_payments": "true" },
            booleanAsStringValueFalse: { "has_card_payments": "false" },
            booleanValueTrue: { "has_card_payments": true },
            booleanValueFalse: { "has_card_payments": false }

        };
        
        expect(_locationViewHelper_.groupProperties(booleanTestCases.intValueTrue)[0].properties[0].value).to.be.true;
        expect(_locationViewHelper_.groupProperties(booleanTestCases.intValueFalse)[0].properties[0].value).to.be.false;

        expect(_locationViewHelper_.groupProperties(booleanTestCases.stringValueTrue)[0].properties[0].value).to.be.true;
        expect(_locationViewHelper_.groupProperties(booleanTestCases.stringValueFalse)[0].properties[0].value).to.be.false;

        expect(_locationViewHelper_.groupProperties(booleanTestCases.booleanAsStringValueTrue)[0].properties[0].value).to.be.true;
        expect(_locationViewHelper_.groupProperties(booleanTestCases.booleanAsStringValueFalse)[0].properties[0].value).to.be.false;

        expect(_locationViewHelper_.groupProperties(booleanTestCases.booleanValueTrue)[0].properties[0].value).to.be.true;
        expect(_locationViewHelper_.groupProperties(booleanTestCases.booleanValueFalse)[0].properties[0].value).to.be.false;

        done();
    });
});
    