var _promisify_ = require("../../src/promises");
var _environment_ = require("../../src/environment");

var util = require("util");

var chai = require('chai');
var expect  = chai.expect;

describe("Promisify", function() {

    before(function(done) {

        _environment_.setAsTest();
        done();
    });

    after(function(done) {

        _environment_.setAsTest();
        done();
    });

    it('can promisify with no argument', function(done) {

        var result = "worked";
        
        var func = function () { return result; };

        var promisifiedFunc = _promisify_.promisify(func);

        promisifiedFunc()
            .then(function (testResult) {
                
                expect(testResult).to.be.eql(result);
                done();
            }).catch(function (error) {

                console.log(error);
            });
    });
    
    it('can promisify with arguments', function(done) {

        var result = "worked good";
        var arg1 = "worked";
        var arg2 = "good";
        
        var func = function (arg1, arg2) { 

            return [arg1, arg2].join(" "); 
        };

        var promisifiedFunc = _promisify_.promisify(func);

        promisifiedFunc(arg1, arg2)
            .then(function (testResult) {

                expect(testResult).to.be.eql(result);
                done();
            }).catch(function (error) {

                console.log(error);
            });
    });

    it('can promise serial', function (done) {

        var valueToResolve = { a: 1 };
        var errorToReject = {
            error: "error"
        };

        var funcWithResolveNull = function () {

            return new Promise(function (resolve, reject) {

                resolve();
            })
        };

        var funcWithResolveWithValue = function () {

            return new Promise(function (resolve, reject) {

                resolve(valueToResolve);
            })
        };

        var funcWithReject = function () {

            return new Promise(function (resolve, reject) {

                reject(errorToReject);
            })
        };

        _promisify_.promiseSerial([
            funcWithResolveNull,
            funcWithResolveWithValue,
            funcWithReject
        ]).then(function (testResult) {

            expect(testResult).to.have.length(3);
            expect(testResult[0]).to.be.undefined;
            expect(testResult[1]).to.equal(valueToResolve);
            expect(testResult[2]).to.equal(errorToReject);


            done();
            }).catch(function (error) {

                console.log(error);
                done();
            });
    });
});
    