var _s3Manager_ = require("../../src/s3-manager");
var _environment_ = require("../../src/environment");
var _core_ = require("../../src/core");

var chai = require('chai');
var expect  = chai.expect;

describe("S3 Manager", function() {

    beforeEach(function(done) {

        _environment_.setAsTest();
        done();
    });

    after(function(done) {

        _environment_.setAsTest();
        done();
    });

    it('can get S3 path for production', function(done) {
    
        _environment_.setAsProduction();

        expect(_s3Manager_.getAWSS3Url()).to.eql("https://otuzbeslik.s3.amazonaws.com");

        done();
    });

    it('can get S3 path for development', function(done) {
    
        _environment_.setAsDevelopment();

        expect(_s3Manager_.getAWSS3Url()).to.eql("https://otuzbeslik.s3.amazonaws.com");

        done();
    });

    it('can get S3 path for test', function(done) {

        expect(_s3Manager_.getAWSS3Url()).to.eql("https://otuzbeslik-test.s3.amazonaws.com");

        done();
    });

    it('can get get absolute cdn url for production', function(done) {

        expect(_s3Manager_.getAbsoluteCdnUrl("relative-url")).to.eql("https://otuzbeslik-test.s3.amazonaws.com/relative-url");
        expect(_s3Manager_.getAbsoluteCdnUrl("/relative-url")).to.eql("https://otuzbeslik-test.s3.amazonaws.com/relative-url");

        done();
    });
    
    it('can get get absolute cdn url for production', function(done) {

        _environment_.setAsProduction();

        expect(_s3Manager_.getAbsoluteCdnUrl("relative-url")).to.eql("https://otuzbeslik.s3.amazonaws.com/relative-url");
        expect(_s3Manager_.getAbsoluteCdnUrl("/relative-url")).to.eql("https://otuzbeslik.s3.amazonaws.com/relative-url");

        done();
    });

    it('can get get absolute cdn url for development', function(done) {

        _environment_.setAsDevelopment();
        expect(_s3Manager_.getAbsoluteCdnUrl("relative-url")).to.eql("https://otuzbeslik.s3.amazonaws.com/relative-url");
        expect(_s3Manager_.getAbsoluteCdnUrl("/relative-url")).to.eql("https://otuzbeslik.s3.amazonaws.com/relative-url");

        done();
    });

    it('can get get absolute cdn url for test', function(done) {

        expect(_s3Manager_.getAbsoluteCdnUrl("relative-url")).to.eql("https://otuzbeslik-test.s3.amazonaws.com/relative-url");
        expect(_s3Manager_.getAbsoluteCdnUrl("/relative-url")).to.eql("https://otuzbeslik-test.s3.amazonaws.com/relative-url");

        done();
    });
});
    