var _text = require("../../src/text");
var _environment = require("../../src/environment");

var chai = require('chai');
var expect  = chai.expect;

describe("Text", function() {

    before(function(done) {

        _environment.setAsTest();
        done();
    });

    after(function(done) {

        _environment.setAsTest();
        done();
    });

    it('can stringify', function(done) {

        var test = {
            a: 1456,
            b: "1456",
            c: "1456 "
        };

        var test2 = {
            test: test,
            b: "1456",
            c: "1456 "
        };

        var outcome = _text.stringify(test);

        expect(outcome).not.to.be.null;
        
        done();
    });

    it('can stringify recursively without any circular issues', function(done) {

        var test1 = {
            a: 1456,
            b: "1456",
            c: "1456 "
        };

        var test2 = {
            test: test1,
            b: "1456",
            c: "1456 "
        };

        var outcome = _text.stringify(test2);

        expect(outcome).not.to.be.null;
        
        done();
    });

    it('can stringify recursively without any circular issues', function(done) {

        var test = {
            test: test,
            b: "1456",
            c: "1456 "
        };

        var outcome = _text.stringify(test);

        expect(outcome).not.to.be.null;
        
        done();
    });

    it('can can convert to numbers only', function(done) {

        expect( _text.numbersOnly("3")).to.eql("3");
        expect( _text.numbersOnly("3sdsdsd")).to.eql("3");
        expect( _text.numbersOnly(null)).to.eql(null);
        expect( _text.numbersOnly("sdsddsd")).to.eql("");
        
        done();
    });

    it('can remove html', function(done) {

        var html = "<div><div><div><div><a>Bla</a></div></div></div></div>"
        expect( _text.textOnly(html)).to.eql("Bla");
        
        done();
    });

    it('can remove punctuation marks', function(done) {

        var test = "Bla, bla. bla";
        expect( _text.textOnly(test, {
            noPunctuationMarks: true
        })).to.eql("Bla bla bla");
        
        done();
    });

    it('can handle turkish characters', function(done) {

        var test = "Ü, Ç. Ğ";
        expect( _text.textOnly(test, {
            noPunctuationMarks: true
        })).to.eql("Ü Ç Ğ");
        
        done();
    });

    it('can remove space', function(done) {

        var test = "Bla bla bla       bla";
        expect( _text.textOnly(test, {
            noSpace: true
        })).to.eql("Bla bla bla bla");
        
        done();
    });
});
    